<?php

/**
 * @file
 *   Contains NewsletterBasic class and
 *   newsletter's module implementation of drupal mail system.
 */

/**
 * Basic Newsletter class.
 */
class NewsletterBasic {

  protected $from;
  protected $format;
  protected $language;

  public function __construct() {
    $this->from = (string) variable_get('newsletter_from')
      ? variable_get('newsletter_from')
      : variable_get('site_mail');
    $this->format = (string) variable_get('newsletter_format', 'html');
    $this->language = language_default();
  }

  /**
   * Basic token replacement of non-dynamic variable.
   *
   * @param $string
   *   The newsletter part we need to token-replace.
   *
   * @return
   *   The token-replaced string.
   */
  protected function replaceTokens($string) {
    $tokens = newsletter_tokens();
    foreach ($tokens as $token => $description) {
      if (strpos($string, $token)) {
        switch ($token) {
          case 'site_name':
            $replacement = variable_get('site_name');
            break;
          case 'site_url':
            global $base_url;
            $replacement =  $base_url;
            break;
          case 'list_name':
            $replacement = isset($this->list->title) ? $this->list->title : '';
            break;
          default :
            $replacement = NULL;
        }
        $string = (is_null($replacement))
          ? $string
          : str_replace("[$token]", $replacement, $string);
      }
    }
    return $string;
  }

  /**
   * Token replacement of subscriber dependent variables.
   *
   * @param $string
   *   The newsletter part we need to token-replace.
   * @param $subscriber
   *   The subscriber's object.
   *
   * @return
   *   The token-replaced string.
   */
  protected function replaceSubscriberTokens($string, $subscriber) {
    $tokens = newsletter_tokens();
    foreach ($tokens as $token => $description) {
      if (strpos($string, $token)) {
        switch ($token) {
          case 'subscription_ip':
            $replacement = $subscriber->ip;
            break;
          case 'subscription_time':
            $replacement = format_date($subscriber->created, 'long');
            break;
          case 'confirmation_url':
            $params = array(
              'absolute' => TRUE,
              'query' => array('hash' => $subscriber->hash),
            );
            $replacement = url('newsletter/confirm', $params);
            break;
          case 'unsubscribe_url':
            $params = array(
              'absolute' => TRUE,
              'query' => array('hash' => $subscriber->hash),
            );
            $replacement = url('newsletter/unsubscribe', $params);
            break;
          case 'subscriber_firstname':
            $replacement = isset($subscriber->firstname) ? $subscriber->firstname : '';
            break;
          case 'subscriber_lastname':
            $replacement = isset($subscriber->lastname) ? $subscriber->lastname : '';
            break;
          case 'subscriber_email':
            $replacement = $subscriber->email;
            break;
          default :
            $replacement = NULL;
        }
        $string = (is_null($replacement))
          ? $string
          : str_replace("[$token]", $replacement, $string);
      }
    }
    return $string;
  }

  /*
   * Returns an image to be added to newsletter body
   * so we can calculate the open rate.
   *
   * @param $hash
   *   Subscriber's personal hash.
   *
   * @return
   *   An html image string.
   */
  protected function addImage($hash) {
    $params = array(
      'absolute' => TRUE,
      'query' => array(
        'hash' => $hash,
        'source' => $this->newsletter->id,
      ),
    );
    $url = url('newsletter/statistics', $params);
    $image = "<img src='$url' width = '1' height = '1' />";
    return $image;
  }

  /**
   * Sets a watchdog message for the just sent newsletter.
   *
   * @param $id
   *   The newsletter id.
   * @param $title
   *   The subscriber's list title.
   * @param $email
   *   The subscriber's email.
   * @param $result
   *   The result from drupal_mail_system.
   */
  protected function setWatchdog($id, $title, $email, $result) {
    watchdog('newsletter', 'Newsletter list with newsletter id @news_id and list name @list_name sent to @sub with status code @status.',
      array(
        '@news_id' => $id,
        '@list_name' => $title,
        '@sub' => $email,
        '@status' => $result,
      ), WATCHDOG_ERROR);
  }

  /**
   * Sends a test newsletter to an email-address.
   * Called from administration Settings tab.
   *
   * @param $to
   *   The e-mail to send the test newsletter to.
   */
  public function sendTest($to) {

    $body = t('This is a test e-mail generated by the Drupal Newsletter module.
      If you succesfully received this e-mail,
      it means that your server is capable of sending e-mails.
      Congratulations!
      You can now create lists, templates and subscribe users to them!');

    $mail_id = 'newsletter_test' . REQUEST_TIME;
    $message = drupal_mail('newsletter', $mail_id , $to , $this->language , array() , $this->from , FALSE);
    $message['subject'] = t('Newsletter test e-mail');
    $message['body'] = $body;
    $message['format'] = $this->format;
    $message['body_format'] = 'plain_text';
    $system = drupal_mail_system('newsletter', $mail_id);
    $message = $system->format($message);
    return $system->mail($message);
  }

  /**
   * Sends email to a subscriber using a basic template (confirmation/welcome/unsubscribe emails).
   *
   * @param $id
   *   The the basic template id of this email.
   * @param $to
   *   The e-mail to send the newsletter confirmation to.
   */
  public function sendBasic($id, $to) {
    $template = db_query('SELECT subject, body, format
      FROM {newsletter_templates}
      WHERE id = :id',
      array(':id' => $id))
      ->fetchAssoc();
    $subscriber = db_query('SELECT *
      FROM {newsletter_subscribers}
      WHERE email = :email',
      array(':email' => $to))
      ->fetchObject();
    $body = $this->replaceTokens($template['body']);
    $body = $this->replaceSubscriberTokens($body, $subscriber);

    $message = drupal_mail('newsletter', $subscriber->id , $to , $this->language , array() , $this->from , FALSE);
    $message['subject'] = $this->replaceTokens($template['subject']);
    $message['body'] = $body;
    $message['body_format'] = $template['format'];

    $message['format'] = $this->format;
    $system = drupal_mail_system('newsletter', $id);
    $message = $system->format($message);
    return $system->mail($message);
  }
}

/**
 * Modify the drupal mail system to send HTML emails.
 */
class NewsletterMailSystem implements MailSystemInterface {
  public function format(array $message) {
    // Format the body according to text format
    // or strip all the tags if email format is plain text
    if (!empty($message['body'])) {
      $message['body'] = ($message['format'] == 'html')
        ? check_markup($message['body'], $message['body_format'])
        : $message['body'];
    }
    if (!empty($message['subject'])) {
      $message['subject'] = ($message['format'] == 'html') ? check_plain($message['subject']) : $message['subject'];
    }
    return $message;
  }
  public function mail(array $message) {
    if (isset($message['headers']['Return-Path']) && !ini_get('safe_mode')) {
      $return_path_set = strpos(ini_get('sendmail_path'), ' -f');
      if (!$return_path_set) {
        $message['Return-Path'] = $message['headers']['Return-Path'];
        unset($message['headers']['Return-Path']);
      }
    }
    if ($message['format']==='html') {
      $message['headers']['Content-Type'] = 'text/html; charset=UTF-8; format=flowed';
    }
    $mimeheaders = array();
    foreach ($message['headers'] as $name => $value) {
      $mimeheaders[] = $name . ': ' . mime_header_encode($value);
    }
    $line_endings = variable_get('mail_line_endings', MAIL_LINE_ENDINGS);
    $mail_subject = mime_header_encode($message['subject']);
    $mail_body = $message['body'];
    $mail_headers = join("\n", $mimeheaders);
    if (isset($message['Return-Path']) && !ini_get('safe_mode')) {
      $mail_result = mail($message['to'], $mail_subject, $mail_body, $mail_headers, '-f ' . $message['Return-Path'] );
    }
    else {
      $mail_result = mail($message['to'], $mail_subject, $mail_body, $mail_headers );
    }
    return $mail_result;
  }
}
