<?php

/**
 * @file
 * Several helper vital functions for newsletter module.
 */


/**
 * Loads up the requested list.
 */
function newsletter_lid_load($lid) {
  return db_query("SELECT * FROM {newsletter_lists} WHERE id = :id",
    array(':id' => $lid))
    ->fetchObject();
}

/**
 * Loads up the requested subscriber.
 */
function newsletter_sid_load($sid) {
  return db_query("SELECT * FROM {newsletter_subscribers} WHERE id = :id",
    array(':id' => $sid))
    ->fetchObject();
}

/**
 * Loads up the requested template.
 */
function newsletter_tid_load($tid) {
  return db_query("SELECT * FROM {newsletter_templates} WHERE id = :id",
    array(':id' => $tid))
    ->fetchObject();
}

/**
 * Loads up the requested template.
 */
function newsletter_nid_load($nid) {
  return db_query("SELECT * FROM {newsletter_newsletters} WHERE id = :id",
    array(':id' => $nid))
    ->fetchObject();
}

/**
 * Checks whether an email is registered to newsletter list or not.
 * @return
 *  boolean TRUE or FALSE.
 */
function newsletter_is_subscribed($email = NULL) {
  if (!isset($email)) {
    global $user;
    $email = isset($user->mail) ? $user->mail : FALSE;
  }
  if ($email) {
    $is_subscribed = db_query('SELECT email FROM {newsletter_subscribers} WHERE email = :email', array(':email' => $email))->fetchField();
    return $is_subscribed ? TRUE : FALSE;
  }
  else {
    return FALSE;
  }
}

/**
 * @return
 *   An array with available newsletter tokens.
 */
function newsletter_tokens() {
  $tokens=array(
    'confirmation_url' => t('The url with the generated hash where the subscriber confirms subscription'),
    'subscription_ip' => t('The IP used when subscriber subscribed to your newsletter list'),
    'unsubscribe_url' => t('eg http://example.com/newsletter/unsubscribe/'),
    'subscriber_firstname' => t("Subscriber's First Name"),
    'subscriber_lastname' => t("Subscriber's Last Name"),
    'subscriber_email' => t('eg subscriber@example.com'),
    'subscription_time' => t('The date this subscriber subscribed to your list'),
    'site_name' => t('eg Example'),
    'site_url' => t('eg http://example.com/'),
    'item_title' => t("The node's title"),
    'item_teaser' => t("The node's teaser"),
    'item_date' => t("The node's created date"),
    'item_author' => t("The node's author"),
    'item_url' => t("The node's url"),
    'list_name' => t("The newsletter's list name"),
    'repeat' => t('The section starting with this token will be replaced for all nodes that are to be sent with this list'),
    '/repeat' => t('End of the repeat section'),
  );
  return $tokens;
}

/**
 * Instantiates a new Newsletter class according to http://drupal.org/node/608152.
 *
 * @return
 *   A newsletter object.
 */
function newsletter_create($lid=NULL, $template_id=NULL, $newsletter_id=NULL) {
  if (isset($template_id)) {
    $newsletter = new NewsletterMailCustom($lid, $template_id, $newsletter_id);
  }
  elseif (isset($lid) && !isset($template_id)) {
    $newsletter = new NewsletterMail($lid);
  }
  else {
    $newsletter = new NewsletterBasic();
  }
  return (object) $newsletter;
}

/**
 * Saves a new newsletter subscriber in database.
 *
 * @param $subscriber
 *   An object conatining the following:
 *    email - Required.
 *    uid - Optional.
 *    firstname - Optional.
 *    lastname - Optional.
 *    age_group - Optional.
 *    job - Optional.
 *    country - Optional.
 *    gender - Optional.
 *    ip - Optional.
 *    confirmed - Required.
 * @param $needs_confirm
 *   Whether a email asking for subscription confirmation should be sent.
 * @return
 *   The id of the newly created subscriber will be created since it the object
 *   is passed by reference.boolean FALSE if the insertion fails.
 */
function newsletter_save_new_subscriber(&$subscriber, $needs_confirm = FALSE) {
  if (!valid_email_address($subscriber->email) || newsletter_is_subscribed($subscriber->email))  {
    return drupal_set_message(t('This is not a valid email.Please provide a valid email'), 'warning');
  }
  $subscriber->id = db_insert('newsletter_subscribers')
    ->fields(array(
      'email' => $subscriber->email,
      'uid' => $subscriber->uid,
      'firstname' => isset($subscriber->firstname) ? $subscriber->firstname : NULL,
      'lastname' => isset($subscriber->lastname) ? $subscriber->lastname : NULL,
      'age_group' => isset($subscriber->age_group) ? $subscriber->age_group : NULL,
      'job' => isset($subscriber->job) ? $subscriber->job : NULL,
      'country' => isset($subscriber->country) ? $subscriber->country : NULL,
      'gender' => isset($subscriber->gender) ? $subscriber->gender : NULL,
      'created' => REQUEST_TIME,
      'ip' => isset($subscriber->ip) ? $subscriber->ip : NULL,
      'confirmed' => $subscriber->confirmed,
      'confirmation_timestamp' => $needs_confirm ? 0 : REQUEST_TIME,
      'hash' =>  drupal_hmac_base64(REQUEST_TIME . $subscriber->email, drupal_get_hash_salt() . ip_address()),
    ))
    ->execute();
  if (!$subscriber->id) {
    return FALSE;
  }
  if ($needs_confirm) {
    $sent = newsletter_create()->sendBasic(1, $subscriber->email);
    $sent
      ? drupal_set_message(t("Thank you for your subscription.An e-mail has sent to your address. Please click to the provided link to prove you own this e-mail and confirm subscription. If you can't find our e-mail please check your spam folder."), 'status')
      : drupal_set_message(t('Thank you for your subscription.Your subscription was recorded but the confirmation mail fail to sent probably due to a server failure.'), 'warning');
  }
}
